@@ -1,3 +1,5 @@
+#!/usr/bin/env python2
+# -*- encoding: utf-8 -*-
 import tarfile
 import tempfile
 import os
@@ -6,18 +8,23 @@ import ConfigParser
 from datetime import datetime
 from getpass import getpass
 import logging
+import shelve
+import json
+import re
 
 import boto
 from boto.s3.key import Key
-import shelve
 import boto.glacier
 import boto.glacier.layer2
 from boto.glacier.exceptions import UnexpectedHTTPResponseError
-from beefish import decrypt, encrypt
+from boto.exception import S3ResponseError
+from beefish import decrypt, encrypt_file
 import aaargh
-import json
+
+__version__ = "0.3.0"
 
 DEFAULT_LOCATION = "us-east-1"
+DEFAULT_DESTINATION = "s3"
 
 app = aaargh.App(description="Compress, encrypt and upload files directly to Amazon S3/Glacier.")
 
@@ -26,7 +33,7 @@ log = logging.getLogger(__name__)
 if not log.handlers:
     logging.basicConfig(level=logging.INFO, format='%(asctime)s %(levelname)s: %(message)s')
 
-config = ConfigParser.SafeConfigParser()
+config = ConfigParser.SafeConfigParser({"default_destination": DEFAULT_DESTINATION})
 config.read(os.path.expanduser("~/.bakthat.conf"))
 
 class glacier_shelve(object):
@@ -70,7 +77,15 @@ class S3Backend:
         con = boto.connect_s3(access_key, secret_key)
         if region_name == DEFAULT_LOCATION:
             region_name = ""
-        self.bucket = con.create_bucket(bucket, location=region_name)
+
+        try:
+            self.bucket = con.get_bucket(bucket)
+        except S3ResponseError, e:
+            if e.code == "NoSuchBucket":
+                self.bucket = con.create_bucket(bucket, location=region_name)
+            else:
+                raise e
+
         self.container = "S3 Bucket: {}".format(bucket)
 
     def download(self, keyname):
@@ -93,7 +108,7 @@ class S3Backend:
         upload_kwargs = {}
         if cb:
             upload_kwargs = dict(cb=self.cb, num_cb=10)
-        k.set_contents_from_file(filename, **upload_kwargs)
+        k.set_contents_from_filename(filename, **upload_kwargs)
         k.set_acl("private")
 
     def ls(self):
@@ -176,7 +191,7 @@ class GlacierBackend:
 
 
     def upload(self, keyname, filename):
-        archive_id = self.vault.create_archive_from_file(file_obj=filename)
+        archive_id = self.vault.concurrent_create_archive_from_file(filename) # , keyname => second argument not on pypi yet
 
         # Storing the filename => archive_id data.
         with glacier_shelve() as d:
@@ -204,7 +219,7 @@ class GlacierBackend:
 
         return None
 
-    def download(self, keyname):
+    def download(self, keyname, job_check=False):
         """
         Initiate a Job, check its status, and download the archive if it's completed.
         """
@@ -246,7 +261,28 @@ class GlacierBackend:
             return encrypted_out
         else:
             log.info("Not completed yet")
-            return None
+            if job_check:
+                return job
+            return
+
+    def retrieve_inventory(self, jobid):
+        """
+        Initiate a job to retrieve Galcier inventory or output inventory
+        """
+        if jobid is None:
+            return self.vault.retrieve_inventory(sns_topic=None, description="Bakthat inventory job")
+        else:
+            return self.vault.get_job(jobid)
+
+    def retrieve_archive(self, archive_id, jobid):
+        """
+        Initiate a job to retrieve Galcier archive or download archive
+        """
+        if jobid is None:
+            return self.vault.retrieve_archive(archive_id, sns_topic=None, description='Retrieval job')
+        else:
+            return self.vault.get_job(jobid)
+
 
     def ls(self):
         with glacier_shelve() as d:
@@ -269,39 +305,90 @@ class GlacierBackend:
 
             self.backup_inventory()
 
-storage_backends = dict(s3=S3Backend, glacier=GlacierBackend)
+STORAGE_BACKEND = dict(s3=S3Backend, glacier=GlacierBackend)
+
+def _get_store_backend(conf, destination=DEFAULT_DESTINATION):
+    if not destination:
+        destination = config.get("aws", "default_destination")
+    return STORAGE_BACKEND[destination](conf)
+
+def _match_filename(filename, destination=DEFAULT_DESTINATION, conf=None):
+    storage_backend = _get_store_backend(conf, destination)
+
+    keys = [name for name in storage_backend.ls() if name.startswith(filename)]
+    keys.sort(reverse=True)
+    return keys
+
+def match_filename(filename, destination=DEFAULT_DESTINATION, conf=None):
+    _keys = _match_filename(filename, destination, conf)
+    regex_key = re.compile(r"(.+)(\d{14})\.tgz(\.enc)?")
+    keys = []
+    for key in _keys:
+        filename, datestr, isenc = re.findall(regex_key, key)[0]
+        keys.append(dict(filename=filename,
+                        backup_date=datetime.strptime(datestr, "%Y%m%d%H%M%S"),
+                        is_enc=bool(isenc)))
+    return keys
+
 
 @app.cmd(help="Backup a file or a directory, backup the current directory if no arg is provided.")
 @app.cmd_arg('-f', '--filename', type=str, default=os.getcwd())
-@app.cmd_arg('-d', '--destination', type=str, default="s3", help="s3|glacier")
-def backup(filename, destination="s3", **kwargs):
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier")
+@app.cmd_arg('-s', '--description', type=str, default=None)
+def backup(filename, destination=None, description=None, **kwargs):
     conf = kwargs.get("conf", None)
-    storage_backend = storage_backends[destination](conf)
+    storage_backend = _get_store_backend(conf, destination)
 
     log.info("Backing up " + filename)
-    arcname = filename.split("/")[-1]
-    stored_filename = arcname + datetime.now().strftime("%Y%m%d%H%M%S") + ".tgz"
+    arcname = filename.strip('/').split('/')[-1]
+    stored_filename = arcname +  datetime.now().strftime("%Y%m%d%H%M%S") + ".tgz"
     
     password = kwargs.get("password")
-    if not password:
+    if password is None:
         password = getpass("Password (blank to disable encryption): ")
+        if password:
+            password2 = getpass("Password confirmation: ")
+            if password != password2:
+                log.error("Password confirmation doesn't match")
+                return
 
     log.info("Compressing...")
-    out = tempfile.TemporaryFile()
-    with tarfile.open(fileobj=out, mode="w:gz") as tar:
-        tar.add(filename, arcname=arcname)
+    with tempfile.NamedTemporaryFile(delete=False) as out:
+        with tarfile.open(fileobj=out, mode="w:gz") as tar:
+            tar.add(filename, arcname=arcname)
+        outname = out.name
 
     if password:
         log.info("Encrypting...")
-        encrypted_out = tempfile.TemporaryFile()
-        encrypt(out, encrypted_out, password)
+        encrypted_out = tempfile.NamedTemporaryFile(delete=False)
+        encrypt_file(outname, encrypted_out.name, password)
         stored_filename += ".enc"
-        out = encrypted_out
+        os.remove(outname)  # remove non-encrypted tmp file
+        outname = encrypted_out.name
 
     log.info("Uploading...")
-    out.seek(0)
-    storage_backend.upload(stored_filename, out)
+    storage_backend.upload(stored_filename, outname)
+    os.remove(outname)
 
+    return True
+
+
+@app.cmd(help="Give informations about stored filename, current directory if no arg is provided.")
+@app.cmd_arg('-f', '--filename', type=str, default=os.getcwd())
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier")
+@app.cmd_arg('-s', '--description', type=str, default=None)
+def info(filename, destination=None, description=None, **kwargs):
+    conf = kwargs.get("conf", None)
+    storage_backend = _get_store_backend(conf, destination)
+    filename = filename.split("/")[-1]
+    keys = match_filename(filename, destination if destination else DEFAULT_DESTINATION)
+    if not keys:
+        return
+    key = keys[0]
+    print "Last backup date: {} ({} versions)".format(key["backup_date"].isoformat(),
+                                                    str(len(keys)))
+
+    return key
 
 @app.cmd(help="Set AWS S3/Glacier credentials.")
 def configure():
@@ -309,8 +396,19 @@ def configure():
     config.set("aws", "access_key", raw_input("AWS Access Key: "))
     config.set("aws", "secret_key", raw_input("AWS Secret Key: "))
     config.set("aws", "s3_bucket", raw_input("S3 Bucket Name: "))
+    while 1:
+        default_destination = raw_input("Default destination ({}): ".format(DEFAULT_DESTINATION))
+        if default_destination:
+            default_destination = default_destination.lower()
+            if default_destination in ("s3", "glacier"):
+                break
+            else:
+                log.error("Invalid default_destination, should be s3 or glacier, try again.")
+        else:
+            default_destination = DEFAULT_DESTINATION
+    config.set("aws", "default_destination", default_destination)
     config.set("aws", "glacier_vault", raw_input("Glacier Vault Name: "))
-    region_name = raw_input("Region Name (" + DEFAULT_LOCATION + "): ")
+    region_name = raw_input("Region Name ({}): ".format(DEFAULT_LOCATION))
     if not region_name:
         region_name = DEFAULT_LOCATION
     config.set("aws", "region_name", region_name)
@@ -321,16 +419,16 @@ def configure():
 
 @app.cmd(help="Restore backup in the current directory.")
 @app.cmd_arg('-f', '--filename', type=str, default="")
-@app.cmd_arg('-d', '--destination', type=str, default="s3", help="s3|glacier")
-def restore(filename, destination="s3", **kwargs):
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier")
+def restore(filename, destination=None, **kwargs):
     conf = kwargs.get("conf", None)
-    storage_backend = storage_backends[destination](conf)
+    storage_backend = _get_store_backend(conf, destination)
 
     if not filename:
         log.error("No file to restore, use -f to specify one.")
         return
 
-    keys = [name for name in storage_backend.ls() if name.startswith(filename)]
+    keys = _match_filename(filename, destination if destination else DEFAULT_DESTINATION)
     if not keys:
         log.error("No file matched.")
         return
@@ -345,7 +443,18 @@ def restore(filename, destination="s3", **kwargs):
             password = getpass()
 
     log.info("Downloading...")
-    out = storage_backend.download(key_name)
+    
+    download_kwargs = {}
+    if kwargs.get("job_check"):
+        download_kwargs["job_check"] = True
+        log.info("Job Check: " + repr(download_kwargs))
+
+    out = storage_backend.download(key_name, **download_kwargs)
+
+    if kwargs.get("job_check"):
+        log.info("Job Check Request")
+        # If it's a job_check call, we return Glacier job data
+        return out
 
     if out and key_name.endswith(".enc"):
         log.info("Decrypting...")
@@ -360,34 +469,40 @@ def restore(filename, destination="s3", **kwargs):
         tar.extractall()
         tar.close()
 
+        return True
+
 
 @app.cmd(help="Delete a backup.")
 @app.cmd_arg('-f', '--filename', type=str, default="")
-@app.cmd_arg('-d', '--destination', type=str, default="s3", help="s3|glacier")
-def delete(filename, destination="s3", **kwargs):
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier")
+def delete(filename, destination=None, **kwargs):
     conf = kwargs.get("conf", None)
-    storage_backend = storage_backends[destination](conf)
+    storage_backend = _get_store_backend(conf, destination)
 
     if not filename:
         log.error("No file to delete, use -f to specify one.")
         return
 
-    keys = [name for name in storage_backend.ls() if name.startswith(filename)]
+    keys = _match_filename(filename, destination, conf)
     if not keys:
         log.error("No file matched.")
         return
 
-    key_name = sorted(keys, reverse=True)[0]
+    # Get first matching keys => the most recent
+    key_name = keys[0]
+
     log.info("Deleting " + key_name)
 
     storage_backend.delete(key_name)
 
+    return True
+
 
 @app.cmd(help="List stored backups.")
-@app.cmd_arg('-d', '--destination', type=str, default="s3", help="s3|glacier")
-def ls(destination="s3", **kwargs):
+@app.cmd_arg('-d', '--destination', type=str, help="s3|glacier")
+def ls(destination=None, **kwargs):
     conf = kwargs.get("conf", None)
-    storage_backend = storage_backends[destination](conf)
+    storage_backend = _get_store_backend(conf, destination)
     
     log.info(storage_backend.container)
 
